"""
This module contains utilities for extracting decision rules and counterfactuals from a DecisionTreeClassifier.
"""
from sklearn.tree import DecisionTreeClassifier


def decision_rule(x, tree: DecisionTreeClassifier):
    """
    Extract the decision rule for the classification of x.
    :param x: instance to classify.
    :param tree: classifier.
    :return:
    """


def counterfactuals():
    pass
